@@ -5,18 +5,26 @@
 ###
 ### Plot-method(s) for fitted hhh4() models
 ###
-### Copyright (C) 2010-2014 Michaela Paul and Sebastian Meyer
-### $Revision: 833 $
-### $Date: 2014-03-12 00:46:56 +0100 (Wed, 12 Mar 2014) $
+### Copyright (C) 2010-2012 Michaela Paul, 2012-2015 Sebastian Meyer
+### $Revision: 1504 $
+### $Date: 2015-10-23 16:52:26 +0200 (Fre, 23. Okt 2015) $
 ################################################################################
 
 
-plot.hhh4 <- function (x, type=c("fitted", "season", "maxEV", "ri"), ...)
+plot.hhh4 <- function (x,
+                       type = c("fitted", "season", "maxEV", "maps", "ri", "neweights"),
+                       ...)
 {
-    cl <- sys.call()
-    cl$type <- NULL
+    stopifnot(x$convergence)
+    cl <- sys.call()  # not match.call() because plotHHH4_season() has no 'x'
+    ## remove the type argument from the call
+    if (is.null(names(cl)) && nargs() > 1L) { # unnamed call plot(x, type)
+        cl[[3L]] <- NULL  # remove the second argument
+    } else {
+        cl$type <- NULL
+    }
     cl[[1L]] <- as.name(paste("plotHHH4", match.arg(type), sep="_"))
-    eval(cl, envir=parent.frame())
+    eval(cl, envir = parent.frame())
 }
 
 
@@ -25,16 +33,25 @@ plot.hhh4 <- function (x, type=c("fitted", "season", "maxEV", "ri"), ...)
 ###
 
 plotHHH4_fitted <- function (x, units = 1, names = NULL,
-                             col = c("orange","blue","grey85","black"),
-                             pch = 19, pt.cex = 0.6,
+                             col = c("grey85", "blue", "orange"),
+                             pch = 19, pt.cex = 0.6, pt.col = 1,
                              par.settings = list(),
                              legend = TRUE, legend.args = list(),
-                             legend.observed = TRUE, ...)
+                             legend.observed = FALSE,
+                             decompose = NULL, meanHHH = NULL, ...)
 {
+    if (is.null(units)) units <- seq_len(x$nUnit)
     if (!is.null(names)) stopifnot(length(units) == length(names))
-    if (length(col) == 3) col <- c(col, "black") else if (length(col) != 4)
-        stop("'col' must be of length 3 or 4") # for backwards compatibility
-
+    
+    col <- if (is.null(decompose) && length(col) == 4) {
+        ## compatibility with surveillance < 1.10-0
+        pt.col <- col[4L]
+        rev(col[-4L])
+    } else {
+        if (isTRUE(decompose)) decompose <- colnames(x$stsObj)
+        plotHHH4_fitted_check_col_decompose(col, decompose, colnames(x$stsObj))
+    }
+    
     if (is.list(par.settings)) {
         par.defaults <- list(mfrow = sort(n2mfrow(length(units))),
                              mar = c(4,4,2,0.5)+.1, las = 1)
@@ -51,37 +68,71 @@ plotHHH4_fitted <- function (x, units = 1, names = NULL,
         legend <- integer(0L)
     }
     if (length(legend) > 0) {
-        legendidx <- 1L + c(if (legend.observed && !is.na(pch)) 0L,
-                            which(c("ne","ar","end") %in% componentsHHH4(x)))
+        legendidx <- 1L + c(
+            if (legend.observed && !is.na(pch)) 0L,
+            if (is.null(decompose)) {
+                which(c("ne","ar","end") %in% componentsHHH4(x))
+            } else seq_along(col))
         default.args <- list(
-            x="topright", col=c(col[4],col[1:3])[legendidx], lwd=6,
-            lty=c(NA,1,1,1)[legendidx], pch=c(pch,NA,NA,NA)[legendidx],
+            x="topright", col=c(pt.col,rev(col))[legendidx], lwd=6,
+            lty=c(NA,rep.int(1,length(col)))[legendidx],
+            pch=c(pch,rep.int(NA,length(col)))[legendidx],
             pt.cex=pt.cex, pt.lwd=1, bty="n", inset=0.02,
-            legend=c("observed","spatiotemporal","autoregressive","endemic")[legendidx]
+            legend=if (is.null(decompose)) {
+                c("observed","spatiotemporal","autoregressive","endemic")[legendidx]
+            } else c("observed", rev(decompose), "endemic")[legendidx]
             )
         legend.args <- modifyList(default.args, legend.args)
     }
 
+    ## get decomposed mean
+    if (is.null(meanHHH)) {
+        meanHHH <- if (is.null(decompose)) {
+            meanHHH(x$coefficients, terms.hhh4(x))
+        } else {
+            decompose.hhh4(x)
+        }
+    }
+    
     ## plot fitted values region by region
     meanHHHunits <- vector(mode="list", length=length(units))
+    names(meanHHHunits) <- if (is.character(units)) units else colnames(x$stsObj)[units]
     for(i in seq_along(units)) {
         meanHHHunits[[i]] <- plotHHH4_fitted1(x, units[i], main=names[i],
-                                              col=col, pch=pch, pt.cex=pt.cex,
-                                              ...)
+                                              col=col, pch=pch, pt.cex=pt.cex, pt.col=pt.col,
+                                              decompose=decompose, meanHHH=meanHHH, ...)
         if (i %in% legend) do.call("legend", args=legend.args)
     }
     invisible(meanHHHunits)
 }
 
+plotHHH4_fitted_check_col_decompose <- function (col, decompose, unitNames)
+{
+    if (is.null(decompose)) {
+        stopifnot(length(col) == 3)
+    } else {
+        nUnit <- length(unitNames)
+        stopifnot(length(decompose) == nUnit,
+                  setequal(decompose, unitNames))
+        if (length(col) == nUnit) {
+            col <- c("grey85", col)  # first color is for "endemic"
+        } else if (length(col) != 1L + nUnit) {
+            warning("'col' should be of length ", 1L + nUnit)
+            col <- c(col[1L], rep_len(col[-1L], nUnit))
+        }
+    }
+    col
+}
+
 
 ### plot estimated component means for a single region
 
 plotHHH4_fitted1 <- function(x, unit=1, main=NULL,
-                             col=c("grey30","grey60","grey85","grey0"),
-                             pch=19, pt.cex=0.6, border=col,
-                             start=x$stsObj@start, end=NULL,
+                             col=c("grey85", "blue", "orange"),
+                             pch=19, pt.cex=0.6, pt.col=1, border=col,
+                             start=x$stsObj@start, end=NULL, xaxis=NULL,
                              xlim=NULL, ylim=NULL, xlab="", ylab="No. infected",
-                             hide0s=FALSE, meanHHH=NULL)
+                             hide0s=FALSE, decompose=NULL, meanHHH=NULL)
 {
     stsObj <- x$stsObj
     if (is.character(unit) &&
@@ -89,6 +140,15 @@ plotHHH4_fitted1 <- function(x, unit=1, main=NULL,
         stop("region '", .unit, "' does not exist")
     if (is.null(main)) main <- colnames(stsObj)[unit]
 
+    col <- if (is.null(decompose) && length(col) == 4) {
+        ## compatibility with surveillance < 1.10-0
+        pt.col <- col[4L]
+        rev(col[-4L])
+    } else {
+        if (isTRUE(decompose)) decompose <- colnames(x$stsObj)
+        plotHHH4_fitted_check_col_decompose(col, decompose, colnames(x$stsObj))
+    }
+
     ## get observed counts
     obs <- observed(stsObj)[,unit]
 
@@ -101,46 +161,172 @@ plotHHH4_fitted1 <- function(x, unit=1, main=NULL,
     end <- if(is.null(end)) tp[length(tp)] else yearepoch2point(end,stsObj@freq)
     stopifnot(start < end)
     tpInRange <- which(tp >= start & tp <= end)            # plot only those
-    tpSubset <- tp[intersect(x$control$subset, tpInRange)] # fitted time points
+    tpInSubset <- intersect(x$control$subset, tpInRange)   # fitted time points
+
+    ## use time indexes as x-values for use of addFormattedXAxis()
+    if (is.list(xaxis)) {
+        tp <- seq_along(obs)
+        start <- tpInRange[1L]
+        end <- tpInRange[length(tpInRange)]
+    }
 
     ## get fitted component means
-    if (is.null(meanHHH))
-        meanHHH <- meanHHH(coef(x,reparamPsi=FALSE), terms(x))
-    meanHHHunit <- sapply(meanHHH, "[", i=TRUE, j=unit)
+    meanHHHunit <- if (is.null(decompose)) {
+        if (is.null(meanHHH))
+            meanHHH <- meanHHH(x$coefficients, terms.hhh4(x))
+        sapply(meanHHH, "[", i=TRUE, j=unit)
+    } else {
+        if (is.null(meanHHH))
+            meanHHH <- decompose.hhh4(x)
+        meanHHH[,unit,c("endemic",decompose)]
+    }
     stopifnot(is.matrix(meanHHHunit), !is.null(colnames(meanHHHunit)),
               nrow(meanHHHunit) == length(x$control$subset))
     meanHHHunit <- meanHHHunit[x$control$subset %in% tpInRange,,drop=FALSE]
+    if (any(is.na(meanHHHunit))) { # -> polygon() would be wrong
+        ## could be due to wrong x$control$subset wrt the epidemic lags
+        ## a workaround is then to set 'start' to a later time point
+        stop("predicted mean contains missing values")
+    }
     
     ## establish basic plot window
     if (is.null(ylim)) ylim <- c(0, max(obs[tpInRange],na.rm=TRUE))
-    plot(c(start,end), ylim, xlim=xlim, xlab=xlab, ylab=ylab, type="n")
+    plot(c(start,end), ylim, xlim=xlim, xlab=xlab, ylab=ylab, type="n",
+         xaxt = if (is.list(xaxis)) "n" else "s")
+    if (is.list(xaxis)) do.call("addFormattedXAxis", c(list(x = stsObj), xaxis))
     title(main=main, line=0.5)
 
     ## draw polygons
-    xpoly <- c(tpSubset[1], tpSubset, tail(tpSubset,1))
-    polygon(xpoly, c(0,meanHHHunit[,"mean"],0),
-            col=col[1], border=border[1])
-    if (x$control$ar$inModel)
-        polygon(xpoly, c(0,rowSums(meanHHHunit[,c("endemic","epi.own")]),0),
-                col=col[2], border=border[2])
-    if (x$control$end$inModel)
-        polygon(xpoly, c(0,meanHHHunit[,"endemic"],0),
-                col=col[3], border=border[3])
-
+    if (is.null(decompose)) {
+        non0 <- which(c("end", "ar", "ne") %in% componentsHHH4(x))
+        plotComponentPolygons(
+            x = tp[tpInSubset],
+            y = meanHHHunit[,c("endemic", "epi.own", "epi.neighbours")[non0],drop=FALSE],
+            col = col[non0], border = border[non0], add = TRUE)
+    } else {
+        non0 <- apply(X = meanHHHunit > 0, MARGIN = 2L, FUN = any)
+        plotComponentPolygons(x = tp[tpInSubset], y = meanHHHunit[, non0, drop = FALSE],
+                              col = col[non0], border = border[non0], add = TRUE)
+    }
+    
     ## add observed counts within [start;end]
     ptidx <- if (hide0s) intersect(tpInRange, which(obs > 0)) else tpInRange
-    points(tp[ptidx], obs[ptidx], col=col[4], pch=pch, cex=pt.cex)
+    points(tp[ptidx], obs[ptidx], col=pt.col, pch=pch, cex=pt.cex)
 
     ## invisibly return the fitted component means for the selected region
     invisible(meanHHHunit)
 }
 
 
+### function which does the actual plotting of the polygons
+
+plotComponentPolygons <- function (x, y, col = 1:6, border = col, add = FALSE)
+{
+    if (!is.vector(x, mode = "numeric") || is.unsorted(x, strictly = TRUE))
+        stop("'x' must be a strictly increasing sequence of time points")
+    stopifnot(nrow(y <- as.matrix(y)) == (nTime <- length(x)))  # y >= 0
+    yc <- if ((nPoly <- ncol(y)) > 1L) {
+        apply(X = y, MARGIN = 1L, FUN = cumsum) # nPoly x nTime
+    } else t(y)
+    
+    if (!add) {
+        ## establish basic plot window
+        plot(range(x), range(yc[nPoly,]), type = "n")
+    }
+    
+    ## recycle graphical parameters
+    col <- rep_len(col, nPoly)
+    border <- rep_len(border, nPoly)
+    
+    ## draw polygons
+    xpoly <- c(x[1L], x, x[length(x)])
+    for (poly in nPoly:1) {
+        polygon(x = xpoly, y = c(0, yc[poly, ], 0),
+                col = col[poly], border = border[poly])
+    }
+}
+
+
+###
+### Maps of the fitted mean components averaged over time
+###
+
+plotHHH4_maps <- function (x,
+    which = c("mean", "endemic", "epi.own", "epi.neighbours"),
+    prop = FALSE, main = which, zmax = NULL,
+    col.regions = hcl.colors(10),
+    labels = FALSE, sp.layout = NULL, ...,
+    map = x$stsObj@map, meanHHH = NULL)
+{
+    which <- match.arg(which, several.ok = TRUE)
+    
+    ## extract district-specific mean components
+    if (is.null(meanHHH)) {
+        meanHHH <- meanHHH(x$coefficients, terms.hhh4(x))
+    }
+    
+    ## select relevant components and convert to an array
+    meanHHH <- simplify2array(
+        meanHHH[c("mean", "endemic", "epi.own", "epi.neighbours")],
+        higher = TRUE)
+    
+    ## convert to proportions
+    if (prop) {
+        meanHHH[,,-1L] <- meanHHH[,,-1L,drop=FALSE] / c(meanHHH[,,1L])
+    }
+    
+    ## select only 'which' components
+    meanHHH <- meanHHH[,,which,drop=FALSE]
+    
+    ## check map
+    map <- as(map, "SpatialPolygonsDataFrame")
+    if (!all(dimnames(meanHHH)[[2L]] %in% row.names(map))) {
+        stop("'row.names(map)' do not cover all fitted districts")
+    }
+    
+    ## average over time
+    comps <- as.data.frame(colMeans(meanHHH, dims = 1))
+    
+    ## attach to map data
+    map@data <- cbind(map@data, comps[row.names(map),,drop=FALSE])
+
+    ## color key range
+    if (is.null(zmax)) {
+        zmax <- if (prop) {
+            ceiling(10*sapply(comps, max))/10
+        } else ceiling(sapply(comps, max))
+        ## sub-components should have the same color range
+        .idxsub <- setdiff(seq_along(zmax), match("mean", names(zmax)))
+        zmax[.idxsub] <- suppressWarnings(max(zmax[.idxsub]))
+    }
+
+    ## add sp.layout item for district labels
+    if (!is.null(layout.labels <- layout.labels(map, labels))) {
+        sp.layout <- c(sp.layout, list(layout.labels))
+    }
+    
+    ## produce maps
+    grobs <- mapply(
+        FUN = function (zcol, main, zmax)
+            spplot(map, zcol = zcol, main = main,
+                   at = seq(0, zmax, length.out = length(col.regions) + 1L),
+                   col.regions = col.regions, sp.layout = sp.layout, ...),
+        zcol = names(comps), main = main, zmax = zmax,
+        SIMPLIFY = FALSE, USE.NAMES = FALSE)
+    if (length(grobs) == 1L) {
+        grobs[[1L]]
+    } else {
+        mfrow <- sort(n2mfrow(length(grobs)))
+        gridExtra::grid.arrange(grobs = grobs, nrow = mfrow[1L], ncol = mfrow[2L])
+    }
+}
+
+
 ###
 ### Map of estimated random intercepts of a specific component
 ###
 
-plotHHH4_ri <- function (x, component, sp.layout = NULL,
+plotHHH4_ri <- function (x, component, labels = FALSE, sp.layout = NULL,
                          gpar.missing = list(col="darkgrey", lty=2, lwd=2),
                          ...)
 {
@@ -151,14 +337,19 @@ plotHHH4_ri <- function (x, component, sp.layout = NULL,
         stop("'component' must (partially) match one of ",
              paste(dQuote(colnames(ranefmatrix)), collapse=", "))
     
-    map <- x$stsObj@map
-    if (is.null(map)) stop("'x$stsObj' has no map")
+    map <- as(x$stsObj@map, "SpatialPolygonsDataFrame")
+    if (length(map) == 0L) stop("'x$stsObj' has no map")
     map$ranef <- ranefmatrix[,comp][row.names(map)]
     
-    if (is.list(gpar.missing) && any(is.na(map$ranef)))
+    if (is.list(gpar.missing) && any(is.na(map$ranef))) {
         sp.layout <- c(sp.layout, 
                        c(list("sp.polygons", map[is.na(map$ranef),]),
                          gpar.missing))
+    }
+    if (!is.null(layout.labels <- layout.labels(map, labels))) {
+        sp.layout <- c(sp.layout, list(layout.labels))
+    }
+    
     spplot(map[!is.na(map$ranef),], zcol = "ranef",
            sp.layout = sp.layout, ...)
 }
@@ -221,14 +412,11 @@ getMaxEV <- function (x)
     if (identical(type <- attr(Lambda, "type"), "zero")) {
         rep.int(0, nrow(x$stsObj))
     } else {
-        sapply(seq_len(nrow(x$stsObj)),
-               if (identical(type, "diagonal")) {
-                   function (t) max(Lambda(t))  # or max(diag(Lambda(t)))
-               } else { # type is NULL
-                   function (t)
-                       eigen(Lambda(t), symmetric=FALSE,
-                             only.values=TRUE)$values[1L]
-               }, simplify=TRUE, USE.NAMES=FALSE)
+        diagonal <- identical(type, "diagonal")
+        vapply(X = seq_len(nrow(x$stsObj)),
+               FUN = function (t)
+                   maxEV(Lambda(t), symmetric = FALSE, diagonal = diagonal),
+               FUN.VALUE = 0, USE.NAMES = FALSE)
     }
 }
 
@@ -267,7 +455,7 @@ createLambda <- function (object)
         function (t) {
             stopifnot(isScalar(t) && t > 0 && t <= nTime)
             Lambda <- meanHHH$ne.exppred[t,] * Wt(t)
-            diag(Lambda) <- meanHHH$ar.exppred[t,]
+            diag(Lambda) <- diag(Lambda) + meanHHH$ar.exppred[t,]
             Lambda
         }
     }
@@ -275,15 +463,43 @@ createLambda <- function (object)
     Lambda
 }
 
+## determine the dominant eigenvalue of the Lambda matrix
+maxEV <- function (Lambda,
+                   symmetric = isSymmetric.matrix(Lambda),
+                   diagonal = FALSE)
+{
+    maxEV <- if (diagonal) {
+        max(Lambda)  # faster than max(diag(Lambda))
+    } else {
+        eigen(Lambda, symmetric = symmetric, only.values = TRUE)$values[1L]
+    }
+    
+    ## dominant eigenvalue may be complex
+    if (is.complex(maxEV)) {
+        if (Im(maxEV) == 0) { # if other eigenvalues are complex
+            Re(maxEV)
+        } else {
+            warning("dominant eigenvalue is complex, using its absolute value")
+            abs(maxEV)
+        }
+    } else {
+        maxEV
+    }
+}
+
+
 
 ###
 ### Plot estimated seasonality (sine-cosine terms) of one or several hhh4-fits
+### either as multiplicative effect on the 'components' (intercept=FALSE)
+### or with intercept=TRUE, which only makes sense if there are no further
+### non-centered covariates and offsets.
 ###
 
 plotHHH4_season <- function (...,
-                             components = c("ar", "ne", "end", "maxEV"),
+                             components = NULL, intercept = FALSE,
                              xlim = NULL, ylim = NULL,
-                             xlab = NULL, ylab = NULL, main = NULL,
+                             xlab = NULL, ylab = "", main = NULL,
                              par.settings = list(), matplot.args = list(),
                              legend = NULL, legend.args = list(),
                              refline.args = list(), unit = 1)
@@ -291,7 +507,13 @@ plotHHH4_season <- function (...,
     objnams <- unlist(lapply(match.call(expand.dots=FALSE)$..., deparse))
     objects <- getHHH4list(..., .names = objnams)
     freq <- attr(objects, "freq")
-    components <- match.arg(components, several.ok=TRUE)
+    components <- if (is.null(components)) {
+        intersect(c("end", "ar", "ne"), unique(unlist(
+            lapply(objects, componentsHHH4), use.names = FALSE)))
+    } else {
+        match.arg(components, choices = c("ar", "ne", "end", "maxEV"),
+                  several.ok = TRUE)
+    }
 
     ## x-axis
     if (is.null(xlim))
@@ -363,21 +585,24 @@ plotHHH4_season <- function (...,
     seasons <- list()
     for(comp in setdiff(components, "maxEV")){
         s2 <- lapply(objects, getSeason, component = comp, unit = unit)
-        seasons[[comp]] <- exp(sapply(s2, function(intseas) do.call("+", intseas)))
+        seasons[[comp]] <- exp(vapply(s2, FUN = if (intercept) {
+            function (intseas) do.call("+", intseas)
+        } else {
+            function (intseas) intseas$season  # disregard intercept
+        }, FUN.VALUE = numeric(freq), USE.NAMES = TRUE))
         do.call("matplot",              # x defaults to 1:freq
-                c(list(seasons[[comp]], xlim=xlim,
-                       ylim=if (is.null(ylim[[comp]]))
-                       c(0,max(1,seasons[[comp]])) else ylim[[comp]],
-                       xlab=xlab, ylab=ylab[[comp]],
-                       main=main[[comp]]), matplot.args))
+                c(list(seasons[[comp]], xlim=xlim, ylim=ylim[[comp]],
+                       xlab=xlab, ylab=ylab[[comp]], main=main[[comp]]),
+                  matplot.args))
         if (match(comp, components) %in% legend)
             do.call("legend", legend.args)
     }
 
     ## plot seasonality of dominant eigenvalue
     if ("maxEV" %in% components) {
-        seasons[["maxEV"]] <- sapply(objects, function(obj)
-                                     getMaxEV_season(obj, unit=unit)$maxEV.season)
+        seasons[["maxEV"]] <- vapply(objects, FUN = function (obj) {
+            getMaxEV_season(obj)$maxEV.season
+        }, FUN.VALUE = numeric(freq), USE.NAMES = TRUE)
         do.call("matplot",
                 c(list(seasons[["maxEV"]], xlim=xlim,
                        ylim=if (is.null(ylim[["maxEV"]]))
@@ -395,10 +620,11 @@ plotHHH4_season <- function (...,
 }
 
 
-#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-# get estimated seasonal pattern in the different components
-#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-getSeason <- function(x, component = c("ar", "ne", "end"), unit = 1)
+#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+# get estimated intercept and seasonal pattern in the different components
+# CAVE: other covariates and offsets are ignored
+#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+getSeason <- function(x, component = c("end", "ar", "ne"), unit = 1)
 {
     stopifnot(inherits(x, c("hhh4","ah4")))
     component <- match.arg(component)
@@ -412,7 +638,7 @@ getSeason <- function(x, component = c("ar", "ne", "end"), unit = 1)
 
     ## get the intercept
     est <- fixef(x, reparamPsi=FALSE)
-    intercept <- est[grep(paste0("^", component, "\\.(1|ri)"), names(est))]
+    intercept <- unname(est[grep(paste0("^", component, "\\.(1|ri)"), names(est))])
     if (length(intercept) == 0) {
         intercept <- 0 # no intercept (not standard)
     } else if (length(intercept) > 1) { # unit-specific intercepts
@@ -429,6 +655,8 @@ getSeason <- function(x, component = c("ar", "ne", "end"), unit = 1)
             stop("cannot handle partially unit-specific seasonality")
         coefSinCos <- coefSinCos[grep(paste0(").",colnames(x$stsObj)[unit]),
                                       names(coefSinCos), fixed=TRUE)]
+        ## drop .unitname-suffix since non-syntactic (cannot reformulate())
+        names(coefSinCos) <- sub("\\)\\..+$", ")", names(coefSinCos))
     }
     if (length(coefSinCos)==0)
         return(list(intercept=intercept, season=rep.int(0,freq)))
@@ -439,23 +667,40 @@ getSeason <- function(x, component = c("ar", "ne", "end"), unit = 1)
                              data=data.frame(t=startseason-1 + seq_len(freq)))
 
     ## Done
-    list(intercept=intercept, season=mmSinCos %*% coefSinCos)
+    list(intercept=intercept, season=as.vector(mmSinCos %*% coefSinCos))
 }
 
 
 #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 # compute dominant eigenvalue of Lambda_t
+# CAVE: no support for Lambda_it
 #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-getMaxEV_season <- function (x, unit = 1)
+getMaxEV_season <- function (x)
 {
     stopifnot(inherits(x, c("hhh4","ah4")))
     nUnits <- x$nUnit
     freq <- x$stsObj@freq
     components <- componentsHHH4(x)
+
+    ## CAVE: this function ignores epidemic covariates/offsets
+    ##       and unit-specific seasonality
+    if (nUnits > 1L && any(c("ar", "ne") %in% components)) {
+        compOK <- vapply(x$control[c("ar","ne")], FUN = function (comp) {
+            terms <- terms(x)$terms
+            epiterms <- terms[,terms["offsetComp",] %in% seq_len(2L),drop=FALSE]
+            identical(as.numeric(comp$offset), 1) &&
+                length(all.vars(removeTimeFromFormula(comp$f))) == 0L &&
+                    all(!unlist(epiterms["unitSpecific",]))
+        }, FUN.VALUE = TRUE, USE.NAMES = FALSE)
+        if (any(!compOK))
+            warning("epidemic components have (unit-specific) ",
+                    "covariates/offsets not accounted for;\n",
+                    "  use getMaxEV() or plotHHH4_maxEV()")
+    }
     
     ## global intercepts and seasonality
-    s2.lambda <- getSeason(x, "ar", unit = unit)
-    s2.phi <- getSeason(x, "ne", unit = unit)
+    s2.lambda <- getSeason(x, "ar")
+    s2.phi <- getSeason(x, "ne")
     
     ## unit-specific intercepts
     ris <- ranef(x, tomatrix=TRUE)
@@ -477,28 +722,24 @@ getMaxEV_season <- function (x, unit = 1)
         Lambda <- if ("ne" %in% components) {
             exp(s2.phi$intercept + ri.phi + if(t==0) 0 else s2.phi$season[t]) * W
         } else matrix(0, nUnits, nUnits)
-        diag(Lambda) <- if ("ar" %in% components) {
-            exp(s2.lambda$intercept + ri.lambda +
+        if ("ar" %in% components) {
+            diag(Lambda) <- diag(Lambda) + exp(s2.lambda$intercept + ri.lambda +
                 if(t==0) 0 else s2.lambda$season[t])
-        } else 0
+        }
         Lambda
     }
 
-    ## calculate the maximum eigenvalue of Lambda_t
-    maxEV <- function(t)
-    {
-        ev <- eigen(createLambda(t), only.values=TRUE)$values
-        if (is.complex(ev)) {
-            warning("Lambda_",t," has complex eigenvalues")
-            ev <- abs(ev)   # if complex, use abs EVs
-        }
-        max(ev)
-    }
-
     ## do this for t in 0:freq
-    maxEV.const <- maxEV(0)
-    maxEV.season <- if (all(c(s2.phi$season, s2.lambda$season) %in% c(-Inf, 0)))
-        rep.int(maxEV.const, freq) else sapply(seq_len(freq), maxEV)
+    diagonal <- !("ne" %in% components)
+    .maxEV <- function (t) {
+        maxEV(createLambda(t), symmetric = FALSE, diagonal = diagonal)
+    }
+    maxEV.const <- .maxEV(0)
+    maxEV.season <- if (all(c(s2.phi$season, s2.lambda$season) %in% c(-Inf, 0))) {
+        rep.int(maxEV.const, freq)
+    } else {
+        vapply(X = seq_len(freq), FUN = .maxEV, FUN.VALUE = 0, USE.NAMES = FALSE)
+    }
 
     ## Done
     list(maxEV.season = maxEV.season,
@@ -516,15 +757,51 @@ getMaxEV_season <- function (x, unit = 1)
 getSeasonStart <- function (object)
 {
     if ((startsample <- object$stsObj@start[2]) == 1) {
-        object$control$data$t[1]
+        object$control$data$t[1L]
     } else {
-        object$control$data$t[1] + object$stsObj@freq-startsample + 1
+        object$control$data$t[1L] + object$stsObj@freq-startsample + 1
     }
 }
 
 
 
+###
+### plot neighbourhood weight as a function of distance (neighbourhood order)
+###
+
+plotHHH4_neweights <- function (x, plotter = boxplot, ...,
+                                exclude = 0, maxlag = Inf)
+{
+    plotter <- match.fun(plotter)
+    
+    ## orders of neighbourhood (o_ji)
+    nbmat <- neighbourhood(x$stsObj)
+    if (all(nbmat %in% 0:1)) {
+        message("'neighbourhood(x$stsObj)' is binary; ",
+                "computing neighbourhood orders ...")
+        nbmat <- nbOrder(nbmat, maxlag=maxlag)
+    }
+
+    ## extract (estimated) weight matrix (w_ji)
+    W <- getNEweights(x)
+    if (is.null(W)) {  # if no spatio-temporal component in the model
+        W <- nbmat
+        W[] <- 0
+    }
+
+    ## draw the boxplot
+    Distance <- factor(nbmat, exclude = exclude)
+    notexcluded <- which(!is.na(Distance))
+    Distance <- Distance[notexcluded]
+    Weight <- W[notexcluded]
+    plotter(Weight ~ Distance, ...)
+}
+
+
+
+###
 ### auxiliary functions
+###
 
 yearepoch2point <- function (yearepoch, frequency, toleft=FALSE)
     yearepoch[1L] + (yearepoch[2L] - toleft) / frequency
