@@ -28,23 +28,21 @@ gpclibCheck <- function (fatal = TRUE)
                           sQuote(paste0("help(", pkgname, ")")), ".")
 
     ## decide if we should run all examples (some take a few seconds)
-    allExamples <- if (interactive()) TRUE else {
-        .withTimings <- Sys.getenv("_R_CHECK_TIMINGS_")
-        withTimings <- !is.na(.withTimings) && nzchar(.withTimings)
-        !withTimings
+    allExamples <- if (interactive()) {
+        TRUE
+    } else { # R CMD check
+        ## only do all examples if a specific environment variable is set
+        ## (to any value different from "")
+        nzchar(Sys.getenv("_R_SURVEILLANCE_ALL_EXAMPLES_"))
+        ## CAVE: testing for _R_CHECK_TIMINGS_ as in surveillance < 1.9-1
+        ## won't necessarily skip long examples for daily checks on CRAN (see
+        ## https://stat.ethz.ch/pipermail/r-devel/2012-September/064812.html
+        ## ). For instance, the daily Windows checks run without timings.
     }
     surveillance.options(allExamples = allExamples)
 }
 
 
-### Function 'base::rep_len' only exists as of R version 3.0.0
-### Define it as a wrapper for base::rep() for older versions
-
-if (getRversion() < "3.0.0" || !exists("rep_len", baseenv())) {
-    rep_len <- function (x, length.out) rep(x, length.out=length.out)
-}
-
-
 
 ###########################
 ### Little helper functions
@@ -70,11 +68,12 @@ clapply <- function (X, FUN, ...)
 
 ### pretty p-value formatting
 
-formatPval <- function (pv, eps = 1e-4)
+formatPval <- function (pv, eps = 1e-4, scientific = FALSE, ...)
 {
     format1 <- function (p)
-        format.pval(p, digits = if (p<10*eps) 1 else 2, eps = eps)
-    sapply(pv, format1)
+        format.pval(p, digits = if (p < 10*eps) 1 else 2, eps = eps,
+                    nsmall = 2, scientific = scientific, ...)
+    vapply(X = pv, FUN = format1, FUN.VALUE = "", USE.NAMES = TRUE)
 }
 
 
